import * as Promise from 'bluebird';
import * as Sequelize from 'sequelize';
import { Synchronizer as Sync } from 'pretur.sync';
import { Model } from 'pretur.spec';
import { I18nBundle } from 'pretur.i18n';
import { Pool } from './pool';
export interface ErrorMessageBuilder<T> {
    (data: T, error: Error): I18nBundle;
}
export interface Insert<T> {
    (transaction: Sequelize.Transaction, item: Sync.Insert<T>, errors: Sync.Error[]): Promise<void>;
}
export interface Update<T> {
    (transaction: Sequelize.Transaction, item: Sync.Update<T>, errors: Sync.Error[]): Promise<void>;
}
export interface Remove {
    (transaction: Sequelize.Transaction, item: Sync.Remove, errors: Sync.Error[]): Promise<void>;
}
export interface Synchronizer<TInsert, TUpdate> {
    (transaction: Sequelize.Transaction, item: Sync.Item, errors: Sync.Error[]): Promise<void>;
}
export interface UnitializedSynchronizer<TInsert, TUpdate> {
    synchronizer: Synchronizer<TInsert, TUpdate>;
    initialize(pool: Pool): void;
}
export interface SynchronizationInterceptor<T> {
    (transaction: Sequelize.Transaction, item: T, errors: Sync.Error[], pool: Pool): Promise<boolean>;
}
export interface BuildSynchronizerOptions<TInsert, TUpdate> {
    insertErrorMessageBuilder?: ErrorMessageBuilder<Sync.Insert<TInsert>>;
    updateErrorMessageBuilder?: ErrorMessageBuilder<Sync.Update<TUpdate>>;
    removeErrorMessageBuilder?: ErrorMessageBuilder<Sync.Remove>;
    insertInterceptor?: SynchronizationInterceptor<Sync.Insert<TInsert>>;
    updateInterceptor?: SynchronizationInterceptor<Sync.Update<TUpdate>>;
    removeInterceptor?: SynchronizationInterceptor<Sync.Remove>;
}
export declare function buildSynchronizer<TInsert, TUpdate>(spec: Model, options?: BuildSynchronizerOptions<TInsert, TUpdate>): UnitializedSynchronizer<TInsert, TUpdate>;
